@@ -1,5 +1,5 @@
 from nPDyn.dataParsers.in16b_fws_scans_reduction import IN16B_FWS
 from nPDyn.dataParsers.in16b_qens_scans_reduction import IN16B_QENS
+from nPDyn.dataParsers.in16b_bats_scans_reduction import IN16B_BATS
 from nPDyn.dataParsers.mantidNexus import processNexus
-from nPDyn.dataParsers.mantidWorkspace import WorkspaceHandler
 from nPDyn.dataParsers.inxConvert import convert
