@@ -7,15 +7,13 @@ Classes
 
 import numpy as np
 
-import xml.etree.cElementTree as ET
+from defusedxml import cElementTree as ET
 
 
 class IN16B_XML:
-    """
-    This class handles xml files containing detector grouping."""
+    """This class handles xml files containing detector grouping."""
 
     def __init__(self, sourceFile, numTubes):
-
         self.sourceFile = ET.parse(sourceFile)
         self.root = self.sourceFile.getroot()
 
@@ -28,7 +26,6 @@ class IN16B_XML:
 
     def _parseXML(self):
         """ Find the groups of channels to be kept for each PSD. """
-
         nameList = []
         valList = []
 
@@ -45,16 +42,16 @@ class IN16B_XML:
         range for each tube.
 
         """
-
-        # Computes the siez of the detector tube.
+        # Computes the size of the detector tube.
         size = int(self.valList[self.nameList == "tube1"][0]) - 1
         size /= self.numTubes
 
+        nbrSD = self.valList.size - self.numTubes
+
         PSDValues = []
-        for idx, val in enumerate(self.valList[2:]):
-            group = val.split("-")
-            group[0] = int(group[0]) - idx * size
-            group[1] = int(group[1]) - idx * size
+        for idx, val in enumerate(self.valList[nbrSD:]):
+            group = np.array(val.split("-")).astype(int)
+            group -= int(idx * size)
             PSDValues.append(group)
 
         return np.array(PSDValues).astype(int)
