@@ -12,10 +12,10 @@ from nPDyn.dataParsers.mantidNexus import processNexus
 def test_import_QENS():
     dataPath = path + "/sample_data/vana_QENS_280K.nxs"
     result = processNexus(dataPath)
-    assert result.intensities.shape == (1, 18, 1024)
+    assert result.shape == (1, 18, 1024)
 
 
 def test_import_FWS():
     dataPath = path + "/sample_data/lys_part_01_FWS.nxs"
     result = processNexus(dataPath, True)
-    assert result.intensities.shape == (21, 18, 4)
+    assert result.shape == (21, 18, 4)
